/**
 *发送验证码方法封装
 * @author  XT
 * @date 2019年7月16日
 */
 async function sendCode() {
  let that = this, phone = that.phone.trim(),
    {handler, natives, config, globalConfig: {APIS, REGEX}, PROMPTS, pop} = that.$app.$def,
    {loggerAsync} = that.$app;

  //添加手机号为空的保护@XT@2018年11月1日
  if (!phone) {
    natives.surface.tip(PROMPTS.AUTH.EMPTY.PHONE);
    return;
  }

  //电信审核拦截
  if (extraControl.interceptTelecomWhenExamining.call(that, phone)) return;

  //电信振铃拦截
  if (extraControl.interceptTelecomWhenRingtone.call(that, phone)) return;

  //运营商版权判断，返回true表示拦截@2019年11月27日
  if (REGEX.telecom.phoneRegex.test(phone) && pop.operatorCopyright({ctx: that, ring: that.ring, phone})) {
    //电信验证码之前就拦截
    close(that);
    return;
  }

  let onlyLogin = that.type !== "crbt" || pop.operatorCopyright({ctx: that, ring: that.ring, phone, tip: false});
  //设振铃的时候，不开任何彩铃相关的业务。
  // if (that.type !== "crbt") onlyLogin = true;

  if (that.messageSend) {
    natives.surface.loading.call(that,true);
    that.messageSend = false;
    let data = {
      cmd: APIS.AUTH.SEND_MESSAGE,
      phone: (new Buffer(phone || '')).toString('base64'),
      dir: onlyLogin ? 1 : config.login.mode,//@2019年1月24日@XT 个人中心不读取配置，固定写1
      type: 6,
      alipay_on: 1// TODO:通知服务端，可以返回支付宝支付，不影响线上
    };

    //写入日志
    loggerAsync(that, {name: 'doSendCheckcode.Before', params: data});

    //验证是否为有效手机号码
    if (!handler.auth.validate.phone.call(that, phone)) {
      that.messageSend = true;
      natives.surface.loading.call(that, false);
      return;
    }


    loggerAsync(that, {name: 'doSendCheckcode', params: data});
    const result = await that.$app.requestAsync({data: data});

    //电信发验证码成功
    extraControl.telecomSendSuccess.call(that, result);

    if (result.s === 1) {
      handler.auth.timeDown.call(that, 60);//开始倒计时
      that.$element('checkCode').focus({focus: true});
      extraControl.mobilecomSendSuccess.call(that, {result, phone});
    } else {
      that.messageSend = true;
      natives.surface.tip(result.msg);
    }
    natives.surface.loading.call(that,false)
  }
};

/**
 * 登录方法
 * @author XT
 * @date 2019年7月16日
 */
 async function login() {
  let that = this,
    {handler, natives, middleware, globalConfig: {REGEX, APIS, BTN_MODE}, config, prePublish, util, pop, PROMPTS,lottery} = that.$app.$def,
    {loggerAsync} = that.$app,
    phone = that.phone.trim(),
    code = that.checkCode.trim();

  //添加手机号为空的保护@XT@2018年11月1日
  if (!phone) {
    natives.surface.tip(PROMPTS.AUTH.EMPTY.PHONE);
    return;
  }
  //添加验证码为空的保护@XT@2018年11月1日
  if (!code) {
    natives.surface.tip(PROMPTS.AUTH.EMPTY.CODE);
    return;
  }

  //电信审核拦截
  if (extraControl.interceptTelecomWhenExamining.call(that, phone)) return;

  //电信振铃拦截
  if (extraControl.interceptTelecomWhenRingtone.call(that, phone)) return;

  //运营商版权判断，返回true表示拦截@2019年11月27日
  if (REGEX.telecom.phoneRegex.test(phone) && pop.operatorCopyright({ctx: that, ring: that.ring, phone})) {
    //电信验证码之前就拦截
    close(that);
    return;
  }


  //设振铃的时候，不开任何彩铃相关的业务。
  let loginSimple = that.type !== "crbt" || pop.operatorCopyright({ctx: that, ring: that.ring, phone, tip: false});


  natives.surface.loading.call(that, true);
  let cmd = '';

  let logName = "alertLogin.doOK";
  const isTelecom = REGEX.telecom.phoneRegex.test(phone);
  const isUnicom = REGEX.unicom.phoneRegex.test(phone);
  const isMobileCom = REGEX.mobilecom.phoneRegex.test(phone);

  console.log("isTelecom：" + isTelecom, "  dir：" + that.dir);
  //电信：动态判断login_crbt或login_order
  if (isTelecom) {
    if (that.dir === 3) cmd = APIS.AUTH.LOGIN_CRBT;
    else if (that.dir === 2) cmd = APIS.AUTH.LOGIN_ORDER;
    else cmd = APIS.AUTH.LOGIN;
  } else if (loginSimple) cmd = APIS.AUTH.LOGIN;
  else cmd = parseInt(config.login.mode) === 1 ? APIS.AUTH.LOGIN : APIS.AUTH.LOGIN_CRBT;

  switch (cmd) {
    case APIS.AUTH.LOGIN: {
      logName = "alertLogin.doOK";
      break;
    }
    case APIS.AUTH.LOGIN_CRBT: {
      logName = "alertLoginCrbt.doOK";
      break;
    }
    case APIS.AUTH.LOGIN_ORDER: {
      logName = "alertLoginOrder.doOK";
      break;
    }
  }

  let data = {
    cmd: cmd,
    phone: (new Buffer(phone)).toString('base64'),
    checkcode: code,
    returnUrl:handler.operator.getUnicomReturnUrl.call(that),
    alipay_on : 1// TODO:通知服务端，可以返回支付宝支付，不影响线上
  };



  //验证手机号和验证码
  if (!handler.auth.validate.phone.call(that, phone) || !handler.auth.validate.code.call(that, code)){
    natives.surface.loading.call(that, false);
    return
  }


  //电信用户在审核状态下，会额外弹窗确认一次。
  const telecomConfirm = await extraControl.telecomExtraConfirm.call(that, cmd);

  //电信用户拒绝支付
  if (!telecomConfirm) {
    natives.surface.loading.call(that, false);
    return;
  }


  //登录日志校验通过后记录@XT@2018年12月25日
  loggerAsync(that, {name: logName, params: data});

  const result = await that.$app.requestAsyncSimple({data});
  const res = JSON.parse(result.data);

  //用户信息存入本地
  if (res.data && res.data.hasOwnProperty('phone')) {
    handler.user.set.call(that, res.data);
    await handler.user.saveLocalAsync.call(that);

    // 登录成功后查询抽奖任务
    try {
      await  lottery.tasks.call(this);
      const exist = await natives.base.shortcut.exist();
      if (exist) await lottery.add.call(this, {type: lottery.type().SHORTCUT});
    } catch (e) {}

  }
  natives.surface.loading.call(that, false);

  switch (res.s) {
    case 1:
    case 80:
    case -80: {

      //移动号码登录成功后查询状态即可，无其他逻辑
      if (isMobileCom) {
        natives.surface.loading.call(that, true);
        middleware.hub.getOrderState(that);
        if (
          that.ring&&
          that.ring.hasOwnProperty('ringid') &&
          that.type === 'alarm' || that.type === 'ringtone') middleware.hub.getDownloadUrl(that, that.ring);
        return;
      }

      that.$dispatch("center-init");//个人中心刷新

      //收藏功能直接返回，不开业务（移动联通）
      if ((that.type === "collect" || that.type === "lot") && isUnicom) {
        // natives.page.back();
        close(that);
        return;
      }

      //运营商版权判断，返回true表示拦截@2019年11月27日
      if (pop.operatorCopyright({ctx: that, ring: that.ring, phone})) {
        close(that);
        return;
      }

      // 拦截电信联通的振铃功能
      if (extraControl.interceptUnicomAndTelecomWhenRingTone.call(that,phone))return;

      //针对电信非彩铃非会员用户，需要两遍验证码；
      if (extraControl.telecomSecondCheck.call(that, phone))return;


      //TODO:待验证，31也排除，彩铃开通中。
      if (res.crbt && !/^(1|30|31)$/.test(res.crbt.s)) {
        try {
          that.page_title = config.crbt.fail.title;
          pop.explain.call(that,util.tools.removeHtmlTag(res.crbt.msg) || config.crbt.fail.content);
          that.btn_mode = BTN_MODE.BACK;
        } catch (e) {
          console.error(e.message);
        }
      }
      else if (res.order) {
        switch (res.order.s) {
          case 22: {
            if (res.order.data && res.order.data.confirmurl) {


              //个人中心登录，需要替换掉 Login页，首页登录，不需替换，因为开通成功后还需回来继续设铃。
              let replace = true;
              if (that.ring && that.ring.hasOwnProperty('ringid')) {
                that.explain = PROMPTS.ORDER.OPEN_VIP_AND_SET_RING;
                that.btn_mode = BTN_MODE.OPEN_ORDER;
                replace = false;
              }
              handler.operator.toUnicomConfirmPage.call(that, {
                url: res.order.data.confirmurl,
                replace,
                result: res.order.data
              });
            }
            break;
          }
          case 1:
          case 20 : {
            if (that.ring&&that.ring.hasOwnProperty('ringid')) {
              that.explain = PROMPTS.ORDER.SET_RING;
              that.btn_mode = BTN_MODE.SET_RING;
              that.tips = '';
            }
            //联通个人中心登录成功，若为是彩是会，则直接返回
            else close(that);// natives.page.back();
            // that.page_title = config.order.succ.title;
            // that.explain = config.order.succ.content;
            // that.btn_mode = BTN_MODE.BACK;
            break;
          }
          default: {
            that.page_title = config.order.fail.title;
            that.explain = util.tools.removeHtmlTag(res.order.msg) || config.order.fail.content;
            that.btn_mode = BTN_MODE.BACK;
            break;
          }
        }
        that.isLogined = true;
      }
      else if (that.ring) {
        const renderTip = () => {

          that.isLogined = true;
          //有铃音数据，设铃
          // 判断彩铃、会员业务状态
          if (handler.operator.isOrder.call(that)) {
            loggerAsync(that, {name: 'alertSetRing'});
            that.explain = PROMPTS.ORDER.SET_RING;
            that.btn_mode = BTN_MODE.SET_RING;
            that.tips = '';
          } else {
            that.page_title = config.order.pop.title;
            that.explain = PROMPTS.ORDER.OPEN_VIP_AND_SET_RING;
            that.tips = isTelecom ? config.order.pop.content : config.user_center.explain.content;
            that.btn_mode = BTN_MODE.OPEN_ORDER;
            loggerAsync(that, {name: 'alertOpenOrder'});
          }

        };
        //联通：login成功单独执行一次set-serve
        if (isUnicom && cmd === APIS.AUTH.LOGIN) that.$app.$def.order.setServe.call(that, {complete: () => renderTip()});
        else renderTip();
      }
      else {
        that.isLogined = true;
        natives.surface.loading.call(that, false);
        if (!prePublish && isUnicom) that.$app.$def.order.setServe.call(that, {replace:true, complete: () =>close(that)});
        else close(that);
      }

      break;
    }

    //-11：电信【彩铃，非会员】
    case -11: {
      //显示二次弹窗

      // [ 鲸云铃声 ] 电信非彩或非会进入单独的开彩铃或开会员页
      if (that.ring && that.ring.hasOwnProperty('ringid') && parseInt(config.user_center.open) !== 0) {
        that.isLogined = true;
        that.explain = PROMPTS.ORDER.OPEN_VIP_AND_SET_RING;
        that.tips = config.order.pop.content;
        that.btn_mode = BTN_MODE.OPEN_ORDER;
      } else natives.page.replace({
        name: "OpenVip",
        params: {checkcode: that.checkCode, phone: that.phone, ring: that.ring || {}}
      });
      break;
    }

    //-12 电信【非彩铃，会员】
    case -12: {
      //显示二次弹窗
      // [ 鲸云铃声 ] 电信非彩或非会进入单独的开彩铃或开会员页
      if (that.ring && that.ring.hasOwnProperty('ringid')  && parseInt(config.user_center.open) !== 0) {
        that.isLogined = true;
        that.explain = PROMPTS.ORDER.OPEN_CRBT;
        that.tips = config.crbt.pop.content;
        that.btn_mode = BTN_MODE.OPEN_CRBT;
      } else natives.page.replace({
        name: "OpenCrbt", params: {checkcode: that.checkCode, phone: that.phone, ring: that.ring || {}}
      });
      break;
    }
    default: {
      that.checkCode = ''; //登录失败以后，清空验证码输入框
      natives.surface.tip(res.msg);
      break;
    }
  }
};




 function sendCodeForOut(phone){
   let {globalConfig, handler, natives} = this.$app.$def;
   const data = {
     cmd: globalConfig.APIS.AUTH.SEND_MESSAGE,
     phone: (new Buffer(phone)).toString('base64'),
     type: 2,
     alipay_on : 1// TODO:通知服务端，可以返回支付宝支付，不影响线上
   };
   if (handler.auth.validate.phone.call(this, phone)) {
     if (!this.messageSend) return;this.messageSend = false;
     this.$app.loggerAsync(this, {name: 'doSendOutCheckcode', params: data});
     natives.surface.loading.call(this, true);
     this.$app.requestAsync({data}).then((result)=>{
       natives.surface.loading.call(this, false);
       if (result.s === 1) {
         handler.auth.timeDown.call(this, 60);//开始倒计时
         this.$element('checkCode').focus({focus: true});
       }
       else {
         setTimeout(() => {this.messageSend = true;}, 50);
         this.codeErrorMessage = result.msg;
       }
     });
   }
 };

/**
 * 一些额外的控制
 * @author XT
 * @date 2019年7月16日
 */
const extraControl={


  // 2019年3月7日 新增需求：电信用户在审核状态下强制关停
  interceptTelecomWhenExamining(phone) {
    let {natives, prePublish, globalConfig: {REGEX}, PROMPTS} = this.$app.$def;
    if (prePublish && REGEX.telecom.phoneRegex.test(phone)) {
      natives.surface.tip(PROMPTS.AUTH.TELECOM_EXAM);
      return true;
    }
    return false;
  },

  //2019年5月17日 电信用户由于没有仅登录的功能，所以振铃暂时拦截。
  interceptTelecomWhenRingtone(phone) {
    let {natives, globalConfig: {REGEX}, PROMPTS} = this.$app.$def;
    if ((this.type === "ringtone" || this.type === 'alarm') && REGEX.telecom.phoneRegex.test(phone)) {
      this.$app.loggerAsync(this, {
        name: "doSetRingtoneResult",
        type: "error",
        msg: PROMPTS.RINGTONE.NOT_SUPPORTED.OPERATOR
      });
      natives.surface.tip(PROMPTS.RINGTONE.NOT_SUPPORTED.OPERATOR);
      return true;
    } else return false;
  },

  //拦截电信联通的振铃功能
  interceptUnicomAndTelecomWhenRingTone(phone) {
    let {natives, globalConfig: {REGEX}, PROMPTS} = this.$app.$def,
      isTelecom = REGEX.telecom.phoneRegex.test(phone),
      isUnicom = REGEX.unicom.phoneRegex.test(phone);
    //联通电信暂时不支持，所以直接返回。
    if ((this.type === "ringtone" || this.type === 'alarm') && (isUnicom || isTelecom)) {
      natives.surface.tip(PROMPTS.RINGTONE.NOT_SUPPORTED.OPERATOR);
      this.$app.loggerAsync(this, {
        name: "doSetRingtone",
        type: "error",
        msg: PROMPTS.RINGTONE.NOT_SUPPORTED.OPERATOR
      });
      close(this);
      // natives.page.back();
      return true;
    } else return false;
  },
  //电信发验证码成功
  telecomSendSuccess(result) {

    let phone = this.phone.trim(), {config, globalConfig: {REGEX}, PROMPTS} = this.$app.$def;
    //电信用户返回  result={s:1,len:4,dir:3,crbt:false,order:false}
    if (REGEX.telecom.phoneRegex.test(phone)) {
      // result.crbt=false;result.order=false;
      // result.crbt=false;
      // result.order=false;
      // result.dir=3;//3代表login_crbt
      if (result.dir) this.dir = result.dir;
      if (result.crbt === false && result.order === false && result.dir !== 1) {
        this.againConfirm = true;
        this.againConfirmTip = PROMPTS.AUTH.TELECOM_AGAIN_CONFIRM;
        this.tips = config.crbt.pop.content;
        console.log(this.againConfirmTip, this.againConfirm, this.tips);
      }
    }
  },

  //移动号码，发送验证码成功后立即发送一条消息给H5，写入settings
  mobilecomSendSuccess({result,phone}){
    let {middleware, globalConfig: {REGEX}} = this.$app.$def;
    if (REGEX.mobilecom.phoneRegex.test(phone) && result.hasOwnProperty('data') && result.data.hasOwnProperty('phone'))
      middleware.hub.setting(this,result.data);
  },

  //电信一次验证码通过，即将发送二次验证码
  telecomSecondCheck(phone){
     let  {handler, natives, globalConfig: {REGEX}, config} = this.$app.$def, {loggerAsync} = this.$app;
    const isTelecom = REGEX.telecom.phoneRegex.test(phone);
    //针对电信非彩铃非会员用户，需要两遍验证码；
    if (isTelecom && this.againConfirm) {
      loggerAsync(this, {name: "alertLoginOrder"});
      this.againConfirm = false;
      handler.auth.resetTimer.call(this);//停止倒计时
      this.checkCode = '';//清空验证码输入框
      this.tips = config.order.pop.content;//提示语改为  开会员提示语
      natives.surface.loading.call(this, false);
      return true
    }
    else return false;
  },

  //针对电信用户，判断是否需要额外显示一次弹窗资费确认。
//返回true，表示继续执行下一步
//返回false，表示用户拒绝
  telecomExtraConfirm(cmd){
    let {natives, globalConfig: {APIS, REGEX}, config, PROMPTS} = this.$app.$def;
    const isTelecom=REGEX.telecom.phoneRegex.test(this.phone);
    if (!isTelecom)return true;
    if (!this.againConfirm) return true;//排除是彩非会和是会非彩的情况，此情况已经有确认页了，无需再次弹窗确认
    // if (!app.prePublish)  {callback(true);return}

    const crbtConfirm=parseInt(config.crbt.again_pop.pop)===1;//是否有彩铃二次确认
    const orderConfirm=parseInt(config.order.again_pop.pop)===1;//是否有会员二次确认
    let tip="";
    switch (cmd) {
      case APIS.AUTH.LOGIN_CRBT:{
        if (crbtConfirm) tip=PROMPTS.ORDER.OPEN_CRBT;
        break;
      }
      case APIS.AUTH.LOGIN_ORDER:{
        if (orderConfirm) {
          if(this.ring && this.ring.hasOwnProperty('ringid') ) tip="是否确认升级会员（6元/月）并设置彩铃？";
          else tip="是否确认升级会员（6元/月）？";
        }
        break;
      }
    }
    if (tip===''){return true}
    return natives.surface.confirmAsync(tip);
  }
};


/**
 * 验证手机号
 * @param phone
 * @return {Promise<void>}
 * @author XT
 * @date 2019年10月21日
 */
async function submitPhone(phone) {
  const {handler, globalConfig: {REGEX}, prePublish, PROMPTS, natives} = this.$app.$def;
  if (!handler.auth.validate.phone.call(this, phone)) return;//查询是否为有效号码
  if (prePublish && REGEX.telecom.phoneRegex.test(phone)) {//查询是否为审核阶段电信
    natives.surface.tip(PROMPTS.AUTH.TELECOM_EXAM);
    return;
  }

  await sendCode.call(this);
  //有效号码，进入下一步，并自动发送验证码
  this.step = 2;
}


async function init() {
  const {handler, ringSet = {}, user = {}, globalConfig: {REGEX, BTN_MODE}, refresh: {yd_status}, middleware, auth_params = {}, PROMPTS, util, natives} = this.$app.$def;
  let isLogin = handler.operator.isLogin.call(this), ring = ringSet, phone = user ? user.phone : '';
  initRing.call(this);//初始化铃声数据
  const r = auth_params && auth_params.r ? auth_params.r : {s: ''};
  const s = r.s;
  if (s === -10) isLogin = false;
  this.isLogined = isLogin;
  this.ring = ring;
  this.phone = user ? user.phone : '';
  this.type = auth_params && auth_params.type ? auth_params.type : "crbt";
  console.log({s, type: this.type});
  this.alreadyLogin =isLogin;
  if (isLogin) {
    const isMobileCom = REGEX.mobilecom.phoneRegex.test(phone);
    if (isMobileCom) {
      if (yd_status) {
        //为true表示至少查询过一次状态了
        this.isCrbt = user.isCrbt;
        this.isVip = user.isVip;
        this.refreshVIPState({crbt: this.isCrbt, order: this.isVip});
      }
      else {
        natives.surface.loading.call(this, true);
        middleware.hub.getOrderState(this);
      }
    }
    else {
      this.isVip=handler.operator.isOrder.call(this);
      await handler.configs.waiting.call(this);
      const config = this.$app.$def.config;
      switch (s) {
        case -10:
          this.isLogined = false;
          if (this.type === "crbt") this.tips = config.login.pop.content;//振铃登录时，底部空白，不显示资费信息
          this.$app.loggerAsync(this, {name: parseInt(config.login.mode) === 1 ? 'alertLogin' : 'alertLoginCrbt'});//异步写入log
          break;
        case -20:
          if (this.ring&&this.ring.hasOwnProperty('ringid')) this.explain = PROMPTS.ORDER.OPEN_VIP_AND_SET_RING;
          else this.explain = PROMPTS.ORDER.OPEN_VIP;
          this.tips = config.user_center.explain.content;
          this.btn_mode = BTN_MODE.OPEN_ORDER;
          this.$app.loggerAsync(this, {name: 'alertOpenOrder'});
          break;
        case -30:
          this.page_title = config.crbt.pop.title;
          // if(that.ring!==null) {
          if (!handler.operator.isOrder.call(this)) this.explain = PROMPTS.ORDER.OPEN_VIP_AND_SET_RING;
          else this.explain = PROMPTS.ORDER.SET_RING;
          // }
          // else that.explain = PROMPTS.ORDER.OPEN_VIP;

          this.tips = config.user_center.explain.content;
          this.btn_mode = BTN_MODE.OPEN_CRBT;
          this.$app.loggerAsync(this, {name: 'alertOpenCrbt'});
          break;
        case -32:
          this.explain = util.tools.removeHtmlTag(r.msg || PROMPTS.ORDER.SET_RING);
          this.btn_mode = BTN_MODE.SET_RING;
          this.$app.loggerAsync(this, {name: 'alertSetRing'});
          break;
        default:
          this.explain = util.tools.removeHtmlTag(r.msg);
          this.btn_mode = BTN_MODE.BACK;
          break;
      }
    }
  }
  else {
    await handler.configs.waiting.call(this);
    const config = this.$app.$def.config;
    if (this.type === "crbt") {
      this.tips = config.login.pop.content;
    }//振铃登录时，底部空白，不显示资费信息
    this.$app.loggerAsync(this, {name: parseInt(config.login.mode) === 1 ? 'alertLogin' : 'alertLoginCrbt'});//异步写入log
  }
}

function  initRing(){
  if (this.ring&&this.ring.hasOwnProperty('ringid')){
    // this.ring=JSON.parse(this.ring);
    if (!this.ring.hasOwnProperty('springid') || !this.ring.hasOwnProperty('picpath') || !this.ring.hasOwnProperty('name'))
      this.$app.$def.data.ringinfo.call(this, this.ring.ringid).then((ring) => {this.ring = ring});
  }
  else{
    //TODO:设铃登录页若不存在铃声信息，则无法设铃
  }
}




//打开设铃/登录页
function open({ctx, params = {}}) {
  ctx.$app.$def["auth_params"] = params;
  if (ctx.hasOwnProperty('auth_init')) ctx.auth_init = true;
  else ctx.$dispatch('auth-to');
}

function close(ctx) {
  ctx.$app.$def["auth_params"] = {};
  if (ctx.hasOwnProperty('auth_init')) ctx.auth_init = false;
  else ctx.$dispatch('auth-back');
}


export default {
  login,
  sendCode,
  sendCodeForOut,
  submitPhone,
  init,
  open,
  close
};